#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QModbusRtuSerialMaster>
#include <QTimer>

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private:
    Ui::MainWindow *ui;

private slots:
    void on_connectButton_clicked();
    void on_serialPortListRefreshButton_clicked();
    void modbusStateChanged(QModbusDevice::State state);
    void replyReady();
    void readAllInputRegisters();

private:
    /**
     * Enum discribing a connection state.
     */
    enum ConnectionState { Connected,
                           Disconnected,
                           Connecting,
                           TimeOut,
                           Error };

private:
    void refreshSerialPortList();
    void refreshStatusBar();
    void setConnectionState(ConnectionState state);
    void parseReply(QModbusReply *reply);

private:
    QModbusRtuSerialMaster *modbusMaster; ///< Modbus device connection.
    ConnectionState connectionState;      ///< Current state of the connection.
    QTimer *timer;                        ///< Timer to periodically read the input registers.
    bool resetConnection;                 ///< Indicates if a connection reset is necessary.
    bool firstRead;                       ///< Indicates if the current read of the input registers
                                          ///< is the first one or not
};

#endif // MAINWINDOW_H
